import { Injectable } from '@nestjs/common';
import { SalesPriceFormulaDataService } from 'src/modules/transaction/sales-price-formula/data/services/sales-price-formula-data.service';
import { TaxDataService } from 'src/modules/transaction/tax/data/services/tax-data.service';
import { TransactionEntity } from '../../entities/transaction.entity';
import { calculateFormula } from 'src/modules/transaction/sales-price-formula/domain/usecases/managers/helpers/calculation-formula.helper';

@Injectable()
export class PriceCalculator {
  constructor(
    private formulaService: SalesPriceFormulaDataService,
    private taxService: TaxDataService,
  ) {}

  private initialValue(tax: any) {
    if (typeof tax !== 'object' || tax === null) {
      return null; // Return null for non-object values
    }

    for (const key in tax) {
      if (tax.hasOwnProperty(key)) {
        tax[key] = null;
      }
    }

    tax['dpp'] = null;

    return tax;
  }

  async calculate(transaction: TransactionEntity) {
    const taxes = await this.taxService.taxKeyValue();
    const formulas = await this.formulaService.profitShareFormula();
    const values = this.initialValue(taxes);

    const dpp = formulas.find((formula) => formula.value_for == 'dpp');
    const dppValue = calculateFormula(
      dpp.formula_string,
      taxes,
      transaction.payment_total_net_profit,
    );

    values['dpp'] = dppValue;

    do {
      const valueFor = this.withNullValue(values);
      const formula = formulas[valueFor];
      let result;

      try {
        result = calculateFormula(
          formula.formula_string,
          values,
          transaction.payment_total_net_profit,
        );
      } catch (error) {}

      values[valueFor] = result;
      values[`${valueFor}_value`] = result;
    } while (this.containsNullValue(values));

    return { dpp_value: dppValue, tax_datas: values };
  }

  containsNullValue(obj) {
    if (typeof obj !== 'object' || obj === null) {
      return obj === null; // Return true if the value itself is null
    }

    for (const key in obj) {
      if (obj.hasOwnProperty(key)) {
        if (this.containsNullValue(obj[key])) {
          return true; // If any nested value is null, return true
        }
      }
    }

    return false; // If no null values are found, return false
  }

  withNullValue(obj) {
    if (typeof obj !== 'object' || obj === null) {
      return null; // Return null for non-object values
    }

    for (const key in obj) {
      if (obj.hasOwnProperty(key)) {
        if (obj[key] === null) {
          return key; // Found a null value, return the key
        } else if (typeof obj[key] === 'object') {
          const nestedKey = this.withNullValue(obj[key]);
          if (nestedKey !== null) {
            return `${key}.${nestedKey}`; // Found null in nested object, return nested key path
          }
        }
      }
    }

    return null; // No null values found, return null
  }
}
